# DOL_template.py 		
# Created:   Wendy Williams 20/10/2012
# Last Edit: Michael Wilby  11/03/2015
#
#       Provide a simple starting point for creating an image noise 
#       reduction script, for practical exercises in the course 'Detection     
#       of Light'.  In addition it provides some important information and
#       lists the most relevant helpful procedures and functions.
#
# CALLING SEQUENCE:
#       > python DOL_template.py
#       or from within ipython: %run DOL_template.py
# ----------------------------------------------------------


# Import libraries
import pyfits as pf  # pyfits for the fits handling 
import numpy as np   # numpy for the arrays

# Note: it is best and safest to import the libraries with an alias to avoid collisions when different libraries contain methods with the same name

# ****************** NOTE **********************
# Python indices always start counting at zero
# an array of 128 pixels has indices from 0 - 127
# ***********************************************

# =========== Some useful Python methods ================
# range
# Python builtin method
# returns an integer array with the specified dimensions.
# Example:  vec = range(5) returns [0,1,2,3,4]
# ---------
# print
# prints output to the standard output device
# Example: print 'The median value is:', x
# ---------
# FOR-loop  [FOR ...: ]
# Note that the whitespace is very important in Python (there are no begin or end statements)
# ...and it makes for more readable code
# Simple way to reformat arrays (although not very elegant)
# Example:  we reformat a square 10x10 pixel array into a string of 100x1 pixels
# for j in range(10):
#   for i in range(10):
#     imlong[i+j*10]= imsquare[i,j]
#   # end for i #
# # end for j #
# ---------
# np.sin(array)
# return the sine of all the elements of the array (input in radians)
# also cos, tan, log, log10, exp, etc...
# ---------
# np.zeros(shape)
# Return a new array of given shape and type, filled with zeros.
# can be used to initialise an empty array
# Example:  im = np.zeros(128)  creates a vector of length 128 
# Example:  im = np.zeros((128,128))  creates a 128x128 array
# Example:  im = np.zeros((128,128,3))  creates a 128x128x3 cube
# See also: ones
# ---------
# np.median
# returns the median value of an array 
# Example: x = np.median(im[:,5]) computes the median of all pixel values
# of array im along the 5th column
# ---------
# np.sum
# returns the sum of the elements of an array.
# Example: x = np.sum(range(5)) sums up the vector [0,1,2,3,4] = 10
# Example: im = np.sum(im,axis=0) sums up along the axis given
# --------- 


# pyfits
# an example showing how to read data from the fits file using pyfits
# and write it to a new file.

import pyfits as pf
import os   # os lets you run system commands and interface with the os

# read the data from fits
data = pf.getdata( 'image1_2015.fits' )
head = pf.getheader( 'image1_2015.fits' )  # we don't really need this now

# do something to the data
data_invert = data.transpose()
data_invert = data_invert*2.

# write as new file
# store the data in a pyfits HDU and that in an HDUList
# writing fits files is slightly more complicated as you need to put the data within a pyfits HDUList object
# this is becomes more important when you need to handle the fits headers as well
hdu = pf.PrimaryHDU( data_invert )
hdulist = pf.HDUList( [hdu] )
# check if output exists? if yes, remove
if os.path.isfile( 'image1_2014_out.fits' ):
  os.system( 'rm image1_2014_out.fits' )
# write to new file
hdulist.writeto( 'image1_2014_out.fits' )

# or use the pyfits convenience method:
#pf.writeto(filename, data, header=None, output_verify='exception', clobber=False, checksum=False)
# use the 
pf.writeto('image1_2014_out2.fits', data_invert, header=head, clobber=True)

# pylab
# an example showing how to plot with matplotblib (aka pylab)

import pylab as pl

x = pl.arange( 0, 2.*pl.pi, pl.pi/1000 )
# arange is like range, but works for floats = array from  0 to 2pi in steps of pi/1000
# arange is in numpy, but pylab inherits all numpy functions

y = 5. + 0.5*x*pl.sin( x )

x2 = pl.arange( 0, 2.*pl.pi, pl.pi/20 )
y2= 5. + 0.5*x2*pl.sin( x2 )

pl.plot( x, y, 'k', label='line' )  # plot in black
pl.plot( x2, y2, 'go', label='points' ) # plot with green circles
pl.legend()
pl.xlabel( '$x$' )  # LaTeX!
pl.ylabel( '$f(x)$' )
pl.title( 'Some function of x' )
pl.savefig( 'dummyplot.png' )

